clear all
Cleft=0.715;                       %[new equilibrium composition at the surface (mole fraction forsterite)]
Cright=0.815;                       %[initial homogenous composition of the olivine(mole fraction forsterite)]
length=1000;                      %[length of the modeled system (*1.e-6 m)]
nx=101;                         %[number of grid points in space]
X=linspace(0,length,nx);      %[definition of an array with the space variables, X (*1.e-6 m)]
dx=X(2)-X(1);                   %[grid size in space (*1.e-6 m)]
Do=2.27*10^(-5);               %[Diffusion coefficient for FeMg in Ol for Fo86, fO2= 1.e-12]
                                %[and T = 1000 C, (1.e-6 m)/s]

%Startkonzentrationen
C=zeros(1,nx);                   
C(1:end)=Cright;          %[Definition of an array with homogeneous starting concentration]

for n=1:nx
    D(n)=Do*10^(3*((1-C(n)-0.14)));     %[Definition of an array with D for each concentration at a given position]
end
dt=0.45*dx^2/D(1);           %[Calculation of the grid size in time (s)]
t=3000;                           %[total time of residence in the magma chamber (days)]
nt=round(t*24*3600/dt);          %[Calculation of the number of iteration for the given total time in days (s)]

%below plotting of simulated profile
%figure(1)
plot(X,C,'black')
xlabel('Distance [microns]')
ylabel('Normalized Conc.')
axis([min(X) max(X) 0 1])
hold on

C(1)=Cleft;                 %[Boundary condition of the diffusion problem: Fixed surface concentration]
R=dt/(dx^2);

%below finite difference scheme to solve diffusion equation where D depends
%on C
for m=1:nt;
    for n=1:nx
         D(n)=Do*10^(3*((1-C(n)-0.14)));
    end

    for n=2:nx-1
        C(n)=C(n)+R*[[(D(n+1)-D(n))*(C(n+1)-C(n))]+[D(n)*(C(n+1)-2*C(n)+C(n-1))]];
    end
end

%below plotting of simulated profile
figure(1)
plot(X,C,'r.')
xlabel('Distance [microns]')
ylabel('mole fraction Fo')
title('Diffusion profile')
axis([min(X) max(X) 0.6 1])



